/*
  Copyright (C) 2001-2002 Laurent Martelli <laurent@aopsys.com>
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  USA */

package org.objectweb.jac.aspects.gui.swing;

import org.objectweb.jac.aspects.gui.DisplayContext;
import org.objectweb.jac.aspects.gui.FieldView;
import org.objectweb.jac.aspects.gui.GuiAC;
import org.objectweb.jac.aspects.gui.ViewFactory;
import org.objectweb.jac.core.rtti.FieldItem;
import org.objectweb.jac.core.rtti.MetaItem;
import org.objectweb.jac.core.rtti.RttiAC;
import org.objectweb.jac.util.Log;
import java.text.DecimalFormat;
import javax.swing.BoxLayout;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.SwingConstants;

/**
 * A Swing viewer component for float values.
 */

public class FloatViewer extends AbstractFieldView
   implements FieldView
{   
   /** Stores the default format of the float */
   protected DecimalFormat format = new DecimalFormat(GuiAC.getFloatFormat());

   protected JLabel label = new JLabel();

   /**
    * Constructs a new float viewer. 
    */
   public FloatViewer(Object value, Object substance, FieldItem field) {
      super(substance,field);
      initFormat();
      setValue(value);
      add(label);
   }

   public FloatViewer() {
      setLayout(new BoxLayout(this,BoxLayout.Y_AXIS));
      label.setAlignmentX((float)1);
      add(label);
   }

   protected void initFormat() {
      MetaItem type = RttiAC.getFieldType(field);
      if (type!=null) {
         format = new DecimalFormat(GuiAC.getFormat(type));
      } else
         format = new DecimalFormat(GuiAC.getFloatFormat(field));
   }

   // FieldView interface

   public void setValue(Object value) {
      if (value!=null)
         label.setText(format.format(((Number)value).doubleValue()));
      else
         label.setText("");
   }

   public void setField(FieldItem field) {
      super.setField(field);
      initFormat();
   }

   protected JComponent getComponent() {
      return label;
   }
}
