/*
  Copyright (C) 2001-2002 Renaud Pawlak, Laurent Martelli
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  USA */

package org.objectweb.jac.aspects.gui.swing;

import org.objectweb.jac.aspects.gui.FieldEditor;
import org.objectweb.jac.aspects.gui.GuiAC;
import org.objectweb.jac.core.rtti.ClassItem;
import org.objectweb.jac.core.rtti.FieldItem;
import org.objectweb.jac.util.Log;
import java.awt.Dimension;
import java.lang.reflect.Constructor;
import javax.swing.BoxLayout;
import javax.swing.JPasswordField;
import javax.swing.JTextField;

/**
 * A Swing editor component for fields values (primitive types).
 */

public class PrimitiveFieldEditor extends AbstractFieldEditor 
   implements FieldEditor 
{
   
   JTextField textField;

   Class type;

   /**
    * Constructs a new primitive field editor. */

   public PrimitiveFieldEditor(Object substance, FieldItem field,
                               Class type, boolean password) 
   {
      super(substance,field);
      this.type = type;
      if (password) 
         textField = new JPasswordField();
      else  
         textField = new org.objectweb.jac.aspects.gui.swing.JTextField();
      // swing "bug" workaround : prevent the textField from extending
      // vertically.
      Dimension minSize = textField.getPreferredSize();
      Dimension maxSize = textField.getMaximumSize();
      textField.setMaximumSize(new Dimension(maxSize.width,minSize.height));


      textField.addFocusListener(this);
      add(textField);
  }

   // FieldEditor interface

   public Object getValue() {
      if (type == int.class || type == Integer.class) {
         return new Integer(textField.getText());
      } else if (type == boolean.class || type == Boolean.class) {
         return( Boolean.valueOf(textField.getText()));
      } else if (type == long.class || type == Long.class) {
         return new Long(textField.getText());
      } else if (type == float.class || type == Float.class) {
         return new Float(textField.getText());
      } else if (type == double.class || type == Double.class) {
         return new Double(textField.getText());
      } else if (type == short.class || type == Short.class) {
         return new Short (textField.getText());
      } else if ( type == byte.class || type == Byte.class) {
         return new Byte(textField.getText());
      } else if (type == char.class || type == Character.class) {
         return new Character(textField.getText().charAt(0));
      } else if (type == String.class) {
         return textField.getText();
      } else {
         try {
            // trying to construct the object from its textual 
            // representation (I think that any class should have
            // a constructor taking a string... this is so helpful...)
            // of course, this will raise an exception most of the time :-(
            Constructor c = type.getConstructor(new Class[] {String.class});
            return c.newInstance( new Object[] {textField.getText()} );
         } catch( Exception e ) {
            e.printStackTrace();
            throw new RuntimeException("Unhandled type "+type.getName());
         }
      }
   }

   public void setValue(Object value) {
      super.setValue(value);
      if( value == null ) value="";
      textField.setText(GuiAC.toString(value));
      Dimension minSize = textField.getPreferredSize();
      minSize.width = 100;
      textField.setMinimumSize(minSize);
   }

   public void setWidth(int width) {
      textField.setColumns(width);
   }

   public void setAutoUpdate(boolean autoUpdate) {
      // TODO
   }

   /**
    * Set the focus on the textField
    */
   public void requestFocus() {
      textField.requestFocus();
   }

}
