/*
  Copyright (C) 2002-2003 Laurent Martelli <laurent@aopsys.com>
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

package org.objectweb.jac.aspects.gui.web;

import org.objectweb.jac.aspects.gui.*;
import org.objectweb.jac.util.Log;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.Arrays;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;

public class AbstractCompositeView extends AbstractView 
   implements CompositeView,HTMLViewer
{
   
   Vector components = new Vector();

   public AbstractCompositeView() {
   }

   public void setContext(DisplayContext context) {
      super.setContext(context);
      // recursively set the display of inner components
      Iterator i = getViews().iterator();
      while (i.hasNext()) {
         View component = (View)i.next();
         component.setContext(context);
      }
   }

   public void addHorizontalStrut(int width) {}
   public void addVerticalStrut(int height) {}

   public void addView(View view, Object extraInfo) {
      view.setContext(context);
      components.add(view);
      view.setParentView(this);
      validate();
   }

   public void addView(View view) {
      addView(view,null);
   }

   public Collection getViews() {
      return components;
   }

   public View getView(Object id) {
      if (id instanceof String)
         return (View)components.get(Integer.parseInt((String)id));      
      else if (id instanceof Integer)
         return (View)components.get(((Integer)id).intValue());
      else
         throw new RuntimeException("getView(): bad id "+id);
   }

   public void removeView(View component)
   {
      components.remove(component);
      validate();
   }

   public void removeAllViews() {
      close();
      components.clear();
   }

   public void close() {
      super.close();
      Log.trace("web.close","closing "+getViews());
      Iterator i = ((Vector)components.clone()).iterator();
      while (i.hasNext()) {
         ((View)i.next()).close();
      }
   }

   protected void add(View component) {
      component.setParentView(this);
      components.add(component);
   }

   public void genDescription(PrintWriter out) {
      if (description!=null) {
         if(!(this instanceof ObjectView && 
              (parentView!=null && parentView.getClass()==Dialog.class))) {
            out.println("<div class=\"description\">"+description+"</div>");
         }
      }
   }

   public void genMessage(PrintWriter out) {
      if(message!=null) {
         String msg=(String)message.invoke(null,new Object[0]);
         out.println("<div class=\"message\">"+msg+"</div>");
      }
   }

   public void genHTML(PrintWriter out) throws IOException {
      Iterator i = components.iterator();
      while (i.hasNext()) {
         HTMLViewer component = (HTMLViewer)i.next();
         out.println("<div class=\""+getClass().getName()+
                     "\" id=\""+((View)component).getLabel()+"\">");
         component.genHTML(out);
         out.println("</div>");
      }
   }
}

