/*
  Copyright (C) 2002-2003 Laurent Martelli <laurent@aopsys.com>
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA */

package org.objectweb.jac.aspects.gui.web;

import java.io.IOException;
import java.io.PrintWriter;
import java.util.Arrays;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Vector;
import org.mortbay.html.Composite;
import org.mortbay.html.Element;
import org.mortbay.html.Link;
import org.mortbay.util.UrlEncoded;
import org.objectweb.jac.aspects.gui.*;
import org.objectweb.jac.aspects.gui.web.html.Button;
import org.objectweb.jac.core.ACManager;
import org.objectweb.jac.core.Wrappee;
import org.objectweb.jac.core.Wrapping;
import org.objectweb.jac.core.rtti.CollectionItem;
import org.objectweb.jac.core.rtti.FieldItem;
import org.objectweb.jac.core.rtti.MethodItem;
import org.objectweb.jac.util.Log;
import org.objectweb.jac.util.Strings;

public abstract class AbstractView implements View {
   
   protected String label;
   protected DisplayContext context;
   protected int width;
   protected int height;
   ViewFactory factory;

   // style used to change display (css for web)
   String style;

   Object[] parameters;
   String type;

   /** row number, for TableCellViewer */
   protected boolean isCellViewer = false;
   protected int row;
   protected int column;
   protected View table;

   public AbstractView() {
   }

   Border viewBorder;
   
   /**
    * Get the value of viewBorder.
    * @return value of viewBorder.
    */
   public Border getViewBorder() {
      return viewBorder;
   }
   
   /**
    * Set the value of viewBorder.
    * @param v  Value to assign to viewBorder.
    */
   public void setViewBorder(Border  v) {
      this.viewBorder = v;
   }
   
   MethodItem message;
   
   /**
    * Get the value of message.
    * @return value of message.
    */
   public MethodItem getMessage() {
      return message;
   }
   
   /**
    * Set the value of message.
    * @param v  Value to assign to message.
    */
   public void setMessage(MethodItem  v) {
      this.message = v;
   }

   protected String description;
   
   /**
    * Get the value of description.
    * @return value of description.
    */
   public String getDescription() {
      return description;
   }
   
   /**
    * Set the value of description.
    * @param v  Value to assign to description.
    */
   public void setDescription(String  v) {
      this.description = v;
   }

   protected View parentView;
   
   /**
    * Get the value of parent.
    * @return value of parent.
    */
   public View getParentView() {
      return parentView;
   }
   
   /**
    * Set the value of parent.
    * @param v  Value to assign to parent.
    */
   public void setParentView(View  v) {
      this.parentView = v;
   }

   public View getRootView() {
      if (parentView==null)
         return this;
      return parentView.getRootView();
   }

   public boolean isDescendantOf(View ancestor) {
      if (this==ancestor)
         return true;
      else if (parentView==null)
         return false;
      else
         return parentView.isDescendantOf(ancestor);
   }

   public AbstractView(ViewFactory factory, DisplayContext context) {
      this.factory = factory;
      this.context = context;
   }

   public void setContext(DisplayContext context) {
      Log.trace("display",2,"setContext on "+this);
      this.context = context;
   }

   public DisplayContext getContext() {
      return context;
   }

   public void setFactory(ViewFactory factory) {
      this.factory = factory;
   }

   public ViewFactory getFactory() {
      return factory;
   }

   public void setLabel(String label) {
      this.label = label;
   }

   public String getLabel() {
      return label;
   }

   public void setWidth(int width) {
      this.width = width;
   }

   public void setHeight(int height) {
      this.height = height;
   }

   public void setType(String type) {
      this.type = type;
   }

   public String getType() {
      return type;
   }

   public void setStyle(String style) {
      this.style = style;
   }

   public String getStyle() {
      return style;
   }

   public void setParameters(Object[] parameters) {
      this.parameters = parameters;
   }
   
   public Object[] getParameters() {
      return parameters;
   }

   public void setFocus(FieldItem field, Object option) {
   }

   public void close() {
      closed = true;
      ((WebDisplay)context.getDisplay()).unregisterView(this);
      Log.trace("view.close","view "+this+" is closed");
   }

   boolean closed=false;

   public boolean isClosed() {
      return closed;
   }

   public void validate() {}


   public boolean equals(Object o) {
      if (!(o instanceof ViewIdentity))
         return false;
      return 
         ( ( type!=null && 
             type.equals(((ViewIdentity)o).getType()) )
           || (type==null && ((ViewIdentity)o).getType()==null ) )
         && ( ( parameters!=null && 
                parameters.equals(((ViewIdentity)o).getParameters()) ) 
              || (parameters==null && ((ViewIdentity)o).getParameters()==null) );
   }

   public int hashCode() {
      return (type!=null?type.hashCode():0) ^ (parameters!=null?parameters.hashCode():0);
   }

   /**
    * Are we in a <FORM> element ?
    */
   protected boolean isInForm() {
      return true;
   }
   
   /**
    * Build an HTML element for an event. It takes into account if we
    * are in a form, and if the browser is MS-IE.
    * @param text text to display for the link
    * @param event the name of the event 
    * @param params additional parameters for the link URL
    * @return an HTML element */
   protected Composite eventURL(String text, String event, String params) {
      JacRequest request = ((WebDisplay)context.getDisplay()).getRequest();
      String parameters = "event="+event+"&amp;source="+getId()+params;
      if (request.isIEUserAgent()) {
         // workaround for MSIE which does not handle <button> the way it should
         Log.trace("web.html","user-agent: "+request.getUserAgent());
         Link link = new Link(
            ((WebDisplay)context.getDisplay()).getServletName()+
            "?"+parameters,
            text);
         link.attribute("onclick","return commitForm(this,'"+parameters+"')");
         return link;
      } else {
         if (isInForm()) {
            Button button = 
               new Button("submit","eventAndAction",parameters);
            button.add(text);
            return button;
         } else {
            return new Link(
               ((WebDisplay)context.getDisplay()).getServletName()+
               "?event="+event+"&amp;source="+getId()+params,
               text);
         }
      } 
   }

   public String getOpenBorder() {
      String s = "";
      if (viewBorder==null) 
         return s;
      if (viewBorder.hasTitle()) {
         s = s+"<div class=\"BORDER_"+Border.i2aStyle(viewBorder.getStyle())+"\">"+
            "<div class=\"label\">"+viewBorder.getTitle()+"</div>";
      } else {
         s = s+"<div class=\"BORDER_"+Border.i2aStyle(viewBorder.getStyle())+"\">";
      }
      return s;
   }

   public String getCloseBorder() {
      if (viewBorder==null) 
         return "";
      return "</div>";
   }

   /**
    * Build the base URL for an event
    * @param event the name of the event
    */
   protected String eventURL(String event) {
      String base = ((WebDisplay)context.getDisplay()).getServletName()+
         "?event="+event+"&amp;source="+getId();
      if (isCellViewer)
         return base+"&amp;tableEventSource="+getId(table)+
            "&amp;row="+row+"&amp;col="+column;
      else
         return base;
   }

   /**
    * Builds an <img> tag for an icon
    *
    * @param icon resource path of icon
    * @param alt alt string for <img> HTML tag
    * @param showAlt if true, return alt if icon is null
    */ 
   protected String iconElement(String icon, String alt) {
      return iconElement(icon,alt,true);
   }

   /**
    * Builds an <img> tag for an icon
    *
    * @param icon resource path of icon
    * @param alt alt string for <img> HTML tag
    * @param showAlt if true, return alt if icon is null
    */ 
   protected String iconElement(String icon, String alt, boolean showAlt) {
      if (Strings.isEmpty(icon)) {
         if (showAlt)
            return alt;
         else 
            return "";
      } else {
         return "<img src=\"resources/"+icon+
            "\" class=\"icon\" alt=\""+alt+"\">";
      }
   }

   protected String getId() {
      return ((WebDisplay)context.getDisplay()).registerView(this);
   }

   protected String getId(View view) {
      return ((WebDisplay)context.getDisplay()).registerView(view);
   }

   // HTMLViewer interface

   /* the style sheet */
   String styleSheet = "resources/org/objectweb/jac/aspects/gui/web/style.css";
   String styleSheetIE = "resources/org/objectweb/jac/aspects/gui/web/ie.css";
   String javascript = "resources/org/objectweb/jac/aspects/gui/web/script.js";

   public void setStyleSheet(String styleSheet) {
      this.styleSheet = styleSheet;
   }

   /**
    * Generate an HTML page, with full headers
    * @see #genBody(PrintWriter)
    */
   protected void genPage(PrintWriter out) throws IOException {
      out.println("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01//EN\">");
      out.println("<html>");
      out.println("  <head>");
      out.println("  <title>"+label+"</title>");
      out.println("  <meta name=\"Author\" content=\"JAC web-gui server\">" );
      out.println("  <script type=\"text/javascript\" src=\""+javascript+"\"></script>");
      genStyleSheets(out,context.getCustomizedView());
      out.println("  </head>");
      out.println("  <body>");
      genBody(out);
      out.println("  </body>");
      out.println("</html>");
   }

   protected void genStyleSheets(PrintWriter out, CustomizedView customized) {
      out.println("  <link rel=\"stylesheet\" type=\"text/css\" "+
                  "href=\""+styleSheet+"\" title=JAC>");
      JacRequest request = ((WebDisplay)context.getDisplay()).getRequest();
      if (request.isIEUserAgent()) {
         out.println("  <link rel=\"stylesheet\" type=\"text/css\" "+
                     "href=\""+styleSheetIE+"\">");
      }
      out.println("  <style type=\"text/css\">");
      Iterator it;
      if (customized!=null) {
         it = customized.getCustomizedGUI().getStyleSheetURLs().iterator();
         while (it.hasNext()) {
            String url = (String)it.next();
            out.println("    @import \""+url+"\";");         
         }
      }
      it = GuiAC.getStyleSheetURLs().iterator();
      while (it.hasNext()) {
         String url = (String)it.next();
         out.println("    @import \""+url+"\";");         
      }
      out.println("  </style>");
   }

   /**
    * Override this method to generate the body of an HTML page.
    * @see #genPage(PrintWriter)
    */
   protected void genBody(PrintWriter out) throws IOException {
      out.println("     <p>Empty page</p>");
   }

   // TableCellViewer interface

   public void setTable(View table) {
      this.table = table;
   }

   public void setRow(int row) {
      this.row = row;
   }

   public void setColumn(int column) {
      this.column = column;
   }

   Hashtable attributes = new Hashtable();
   public void setAttribute(String name, String value) {
      attributes.put(name,value);
   }
   protected void printAttributes(PrintWriter out) {
      Iterator it = attributes.entrySet().iterator();
      while (it.hasNext()) {
         Map.Entry entry = (Map.Entry)it.next();
         out.write(" "+entry.getKey()+"=\""+entry.getValue()+"\"");
      }
   }

}
