/*
  Copyright (C) 2002-2003 Laurent Martelli <laurent@aopsys.com>
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

package org.objectweb.jac.aspects.gui.web;

import org.objectweb.jac.aspects.gui.*;
import org.objectweb.jac.core.Collaboration;
import org.objectweb.jac.util.Log;
import org.objectweb.jac.util.Semaphore;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.Map;
import javax.servlet.http.HttpServletResponse;

/**
 * An HTML page containing a View, and an OK and a close Button
 */
public class Dialog extends AbstractPage 
   implements DialogView, DialogListener
{
   boolean ok = false;
   Semaphore semaphore = new Semaphore();
   String description;
   HttpServletResponse response;
   JacRequest jacRequest;
   /** Stores context attributes at creation time */
   Map attributes;

   /**
    * @param view the view to embed in the dialog
    * @param parent the parent window of the dialog
    * @param title the title
    * @param description description of the view
    */
   public Dialog(View view, Object parent,
                 String title, String description) {
      super(view,false);
      this.description = description;
      attributes = Collaboration.get().getAttributes();   
      attributes.remove("gui.web.request");
      attributes.remove("gui.web.response");
      //System.out.println("Stored attributes: "+attributes);
   }

   public HttpServletResponse getResponse() {
      return response;
   }

   public JacRequest getRequest() {
      return jacRequest;
   }

   // DialogView interface

   public boolean waitForClose() throws TimeoutException {
      Log.trace("gui.timeout","waiting for "+this+" to be closed "+
                GuiAC.dialogTimeout+"ms");
      if (!semaphore.acquire(GuiAC.dialogTimeout)) {
         Log.trace("gui.timeout","Dialog timedout: "+this);
         throw new TimeoutException(this);
      }
      Log.trace("gui.events","closed "+this+" -> "+ok);
      return ok;
   }

   public View getContentView() {
      return view;
   }

   // HTMLViewer interface

   public void genBody(PrintWriter out) throws IOException {
      out.println("<div class=\"dialog\">");
      if (description!=null)
         out.println("<div class=\"description\">"+description+"</div>");
      out.println("  <form action=\""+
                  ((WebDisplay)context.getDisplay()).getServletName()+"\" "+
                  "method=\"post\" "+
                  "enctype=\"multipart/form-data\">");
      ((HTMLViewer)view).genHTML(out);
      out.println("  <div class=\"actions\">");
      out.println("    <input type=\"hidden\" name=\"source\" value=\""+getId()+"\">");
      out.println("    <input type=\"hidden\" name=\"event\" value=\"onClose\">");
      out.println("    <input class=\"button\" type=\"submit\" name=\"onOK\" value=\""+GuiAC.getLabelOK()+"\">");
      out.println("    <input class=\"button\" type=\"submit\" name=\"onCancel\" value=\""+GuiAC.getLabelCancel()+"\">");
      out.println("  </div>");
      out.println("  </form>");
      out.println("</div>");
   }

   // DialogListener interface

   public void restoreContext() {
      Log.trace("gui.events","Restoring attributes: "+attributes.keySet());
      Collaboration.get().setAttributes(attributes);
   }

   public void onOK(JacRequest request) {
      Log.trace("gui.events",this+".onOK");
      restoreContext();
      WebDisplay display = (WebDisplay)context.getDisplay();
      response = display.getResponse();
      jacRequest = display.getRequest();
      display.closeWindow(this);
      ok = true;
      semaphore.release();
   }

   public void onValidate(JacRequest request){
      restoreContext();
      WebDisplay.readValues(context,request,true);
   }

   public void onCancel() {
      Log.trace("gui.events",this+".onCancel");
      ok = false;
      Collaboration collab = Collaboration.get();
      try {
         WebDisplay display = (WebDisplay)context.getDisplay();
         response = display.getResponse();
         jacRequest = display.getRequest();
         collab.addAttribute(GuiAC.NO_COMMIT,Boolean.TRUE);
         display.closeWindow(this);
      } finally {
         collab.addAttribute(GuiAC.NO_COMMIT,null);
         semaphore.release();
      }
   }
}
