/*
  Copyright (C) 2001-2002 Laurent Martelli <laurent@aopsys.com>
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

package org.objectweb.jac.aspects.gui.web;

import org.objectweb.jac.aspects.gui.GuiAC;
import org.objectweb.jac.core.rtti.FieldItem;
import org.objectweb.jac.core.rtti.MetaItem;
import org.objectweb.jac.core.rtti.RttiAC;
import org.objectweb.jac.util.Log;
import java.io.PrintWriter;
import java.text.DecimalFormat;

public class FloatViewer extends AbstractFieldView 
   implements HTMLViewer
{
   Object value;
   boolean isCellRenderer = false;

   /** Stores the default format of the float */
   protected DecimalFormat format = new DecimalFormat(GuiAC.getDateFormat());

   public FloatViewer(Object value, Object substance, FieldItem field) {
      super(substance,field);
      initFormat();
      setValue(value);
   }

   public FloatViewer() {
      isCellRenderer = true;
   }

   protected void initFormat() {
      MetaItem type = RttiAC.getFieldType(field);
      String formatString = null;
      if (type!=null)
         formatString = GuiAC.getFloatFormat(type);
      else
         formatString = GuiAC.getFloatFormat(field);

      if (formatString!=null) 
         format = new DecimalFormat(formatString);
      else
         Log.error("No format for "+field);
   }

   public void setValue(Object value) {
      this.value = value;
   }

   public void setField(FieldItem field) {
      super.setField(field);
      initFormat();
   }

   public void genHTML(PrintWriter out) {
      if (value!=null) {
         if (isCellRenderer) 
            out.print("<div style=\"text-align: right\">");
         out.print(format.format(((Number)value).doubleValue()));
         if (isCellRenderer) 
            out.print("</div>");         
      }
      else
         out.print("");
   }
}
