/*
  Copyright (C) 2003 <laurent@aopsys.com>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA */

package org.objectweb.jac.aspects.idGen;

import org.objectweb.jac.core.AspectComponent;
import org.objectweb.jac.core.Collaboration;
import org.objectweb.jac.core.Interaction;
import org.objectweb.jac.core.NameRepository;
import org.objectweb.jac.core.Wrapper;
import org.objectweb.jac.core.rtti.ClassItem;
import org.objectweb.jac.core.rtti.FieldItem;
import org.objectweb.jac.util.Log;


public class IdGenAC extends AspectComponent {

   public static final String COUNTER = "IdGenAC.COUNTER";
   public static final String ID_FIELD = "IdGenAC.ID_FIELD";

   public void genId(ClassItem cl, 
                     String counter, String fieldName) {
      cl.setAttribute(COUNTER,counter);
      cl.setAttribute(ID_FIELD,cl.getField(fieldName));
      pointcut("ALL", cl.getName(), "CONSTRUCTORS",
               IdGenWrapper.class.getName(), "genId", null, false);
   }

   Counters counters;
   protected Counters getCounters() {
      if (counters==null) {
         if (countersName!=null) {
            counters = (Counters)NameRepository.get().getObject(countersName);
            if (counters==null) {
               Log.error("IdGenAC: No object named "+countersName);
            }
         }
      }
      return counters;
   }

   String countersName = "counters0";
   public void setCountersName(String countersName) {
      this.countersName = countersName;
   }

   public class IdGenWrapper extends Wrapper {
      public IdGenWrapper(AspectComponent ac) {
         super(ac);
      }

      public Object genId(Interaction interaction) {
         Object result = proceed(interaction);
         Log.trace("id","generating id for "+interaction.wrappee);
         ClassItem cl = interaction.getClassItem();
         FieldItem field = (FieldItem)cl.getAttribute(ID_FIELD);
         if (Collaboration.get().getAttribute("PersistenceAC.RESTORE")==null) {
            String counterName = (String)cl.getAttribute(COUNTER);
            Counters counters = getCounters();
            if (counters!=null) {
               long id = counters.genId(counterName);
               Log.trace("id","    -> "+id);
               try {
                  if (field.getType()==String.class)
                     field.setThroughWriter(interaction.wrappee, Long.toString(id));
                  else
                     field.setThroughWriter(interaction.wrappee, new Long(id));
               } catch (IllegalAccessException e) {
                  Log.error("Failed to to set field "+field+" for "+interaction.wrappee);
               }
            } else {
               Log.trace("id","    No counters object");               
            }
         } else {
            Log.trace("id","    skipping id generation for "+interaction.wrappee);
         }
         return result;
      }
   }
}
