/*
  Copyright (C) 2002 Renaud Pawlak <renaud@aopsys.com>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

package org.objectweb.jac.aspects.integrity;

import org.objectweb.jac.core.AspectComponent;
import org.objectweb.jac.core.Collaboration;
import org.objectweb.jac.core.Interaction;
import org.objectweb.jac.core.NameRepository;
import org.objectweb.jac.core.ObjectRepository;
import org.objectweb.jac.core.Wrapper;
import org.objectweb.jac.core.rtti.ClassItem;
import org.objectweb.jac.core.rtti.CollectionItem;
import org.objectweb.jac.core.rtti.FieldItem;
import org.objectweb.jac.util.Log;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;

/**
 * This wrapper manages repository collections.
 */

public class RepositoryWrapper extends Wrapper {
   Object repository;
   String repositoryName;
   CollectionItem collection;
   FieldItem field;

   public RepositoryWrapper(AspectComponent ac,
                            String repositoryName,
                            CollectionItem collection,
                            FieldItem field) {
      super(ac);
      this.repositoryName = repositoryName;
      this.collection = collection;
      this.field = field;
   }
   
   /**
    * Adds the object added to field to the repository
    */
   public Object addToRepository(Interaction interaction) {
      Object ret = proceed(interaction);
      if (Collaboration.get().getAttribute(IntegrityAC.DISABLE_ADD_TO_REPOSITORY)!=collection) {
         Log.trace("integrity.repository",
                   "checking for repository on "+interaction.method);      
         if (repository==null)
            repository = NameRepository.get().getObject(repositoryName);
         Object toAdd = interaction.args[0];
         
         if (toAdd!=null && repository!=null && 
             !collection.getActualCollection(repository).contains(toAdd)) {
            Log.trace("integrity.repository","addToRepository("+repositoryName+")");
            collection.addThroughAdder(repository,toAdd);
         }
      } else {
         Log.trace("integrity.repository",collection+" disabled");
      }
      return ret;
   }

   /**
    * Disables addToRepository
    */
   public Object doAddToRepository(Interaction interaction) {
      Collaboration.get().addAttribute(IntegrityAC.DISABLE_ADD_TO_REPOSITORY,collection);
      Log.trace("integrity.repository","doAddToRepository "+collection);
      try {
         return proceed(interaction);
      } finally {
         Collaboration.get().removeAttribute(IntegrityAC.DISABLE_ADD_TO_REPOSITORY);
      }
   }

   public Object removeFromRepository(Interaction interaction) {
      Log.trace("integrity.constraint",
                "removeFromRepository("+repositoryName+")");
      Object toRemove = interaction.args[0];
      FieldItem target = collection!=null?collection:field;
      List constraints = 
         (List)target.getAttribute(IntegrityAC.CONSTRAINTS);
      if (constraints==null)
         return proceed(interaction);
      Iterator it = constraints.iterator(); 
      while (it.hasNext()) {
         Constraint constraint = (Constraint)it.next();
         Collection objects = 
            ObjectRepository.getObjectsWhere(
               (ClassItem)constraint.relation.getParent(),
               constraint.relation,toRemove);
         Log.trace("integrity.constraint","   "+constraint+" => "+objects);
         switch (constraint.constraint) {
            case Constraint.DELETE_CASCADE:
               break;
            case Constraint.SET_NULL:
               Iterator it2 = objects.iterator();
               while (it2.hasNext()) {
                  Object substance = it2.next();
                  Log.trace("integrity.constraint","      "+
                            "SET_NULL "+substance+"."+constraint.relation);
                  try {
                     if (constraint.relation instanceof CollectionItem) {
                        ((CollectionItem)constraint.relation).
                           removeThroughRemover(substance,toRemove);
                     } else {
                        constraint.relation.setThroughWriter(substance,null);
                     }
                  } catch (Exception e) {
                     Log.error("Failed to enforce SET_NULL constraint for "+
                               constraint.relation+" on "+substance);
                     e.printStackTrace();
                  }
               }
               break;
            case Constraint.FORBIDDEN:
               if (objects.size()>0)
                  throw new RuntimeException(
                     "Cannot delete "+toRemove+
                     " from "+target.getLongName()+
                     " because of constraint "+constraint);
               break;
            default:
               Log.warning("integrity","Unknown constraint type "+
                           constraint.constraint);
         }
      }

      return proceed(interaction);
   }

}
