/*
  Copyright (C) 2001-2003 Laurent Martelli <laurent@aopsys.com>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

package org.objectweb.jac.aspects.persistence;

import org.objectweb.jac.util.Log;
import org.objectweb.jac.util.Strings;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;


/**
 * Implements the storage to store within a PostgreSQL database.
 */

public class PostgresStorage extends SQLStorage {

   /**
    * Creates a new storage for a PostgreSQL database.
    *
    * @param database the database. It may take on eof the following form:
    * <ul>
    *   <li>database</li>
    *   <li>//host/database</li>
    *   <li>//host:port/database</li>
    * </ul>
    * @param user the user name
    * @param password the password for this user
    */

   public PostgresStorage(String database, String user, String password) 
      throws SQLException {
      try {
         Class.forName("org.postgresql.Driver");
      } catch (ClassNotFoundException e) {
         Log.error("failed to load postgresql JDBC driver : " +e);
         return;
      }
      try {
         Connection db;
         db = DriverManager.getConnection("jdbc:postgresql:"+database,user,password);
         setConnection(db);
      } catch (SQLException e) {
         Log.error("failed to connect to the database with "+
                   "jdbc:postgresql:"+database+","+user+","+password+": "+e);
         e.printStackTrace();
      }
   }

   public long getNextVal(String sequence) throws Exception {
      return getLong("select nextval('"+sequence+"')");
   }

   public String newName(String className) throws Exception {
      className = Strings.getShortClassName(className).toLowerCase();
      String seq = className+"_seq";
      try {
         execute("create sequence \""+seq+"\"");
      } catch (SQLException e) {
         // <HACK>An excpetion is thrown if the sequence already exists
         // We'd better test the existence of the sequence</HACK>
      }
      return className+(getNextVal(seq)-1);
   }
}
